@@ -23,28 +23,63 @@
 #ifndef _DNRD_COMMON_H_
 #define _DNRD_COMMON_H_
 
+#include <sys/types.h>
 #include <netinet/in.h>
 #include <syslog.h>
 #include <semaphore.h>
+#include "domnode.h"
 
-#define MAX_SERV           5          /* maximum number of DNS servers */
+/* default chroot path. this might be redefined in compile time */ 
+#ifndef CHROOT_PATH
+#define CHROOT_PATH "/etc/dnrd"
+#endif 
+
+#ifndef CONFIG_FILE
+#define CONFIG_FILE "dnrd.conf"
+#endif
+
+/* Set the default timeout value for select in seconds */
+#ifndef SELECT_TIMEOUT
+#define SELECT_TIMEOUT 1
+#endif
+
+/* Set the default timeout value for forward DNS. If we get no
+ * response from a DNS server within forward_timeout, deactivate the
+ * server.  note that if select_timeout is greater than this, the
+ * forward timeout *might* increase to select_timeout. This value
+ * should be >= SELECT_TIMEOUT
+ */
+#ifndef FORWARD_TIMEOUT
+#define FORWARD_TIMEOUT 20
+#endif
+
+/* not used yet */
+#ifndef FORWARD_RETRIES
+#define FORWARD_RETRIES 5
+#endif
+
+/* only check if any server are to be reactivated every
+ * REACTIVATE_INTERVAL seconds
+ */
+#define REACTIVATE_INTERVAL 10
 
 struct dnssrv_t {
-    int                    sock;      /* for communication with server */
-    struct sockaddr_in     addr;      /* IP address of server */
-    char*                  domain;    /* optional domain to match.  Set to
+  int                    sock;      /* for communication with server */
+  struct sockaddr_in     addr;      /* IP address of server */
+  char*                  domain;    /* optional domain to match.  Set to
 					 zero for a default server */
+  
 };
 
+
 extern const char*         version;   /* the version number for this program */
 extern const char*         progname;  /* the name of this program */
 extern int                 opt_debug; /* debugging option */
 extern const char*         pid_file; /* File containing current daemon's PID */
 extern int                 isock;     /* for communication with clients */
 extern int                 tcpsock;   /* same as isock, but for tcp requests */
-extern struct dnssrv_t     dns_srv[]; /* DNS server information struct */
-extern int                 serv_act; /* index into dns_srv for active server */
-extern int                 serv_cnt;  /* number of DNS servers */
+extern int                 select_timeout; /* select timeout in seconds */
+extern int                 forward_timeout; /* timeout for forward DNS */
 extern struct sockaddr_in  recv_addr; /* address on which we receive queries */
 extern uid_t               daemonuid; /* to switch to once daemonised */
 extern gid_t               daemongid; /* to switch to once daemonised */
@@ -52,6 +87,14 @@ extern int                 gotterminal;
 extern char		   master_param[200];
 extern sem_t               dnrd_sem;  /* Used for all thread synchronization */
 
+extern char                chroot_path[512];
+extern char                config_file[];
+extern domnode_t           *domain_list;
+
+extern int                 reactivate_interval;
+extern int                 ignore_inactive_cache_hits; 
+
+
 /* kill any currently running copies of dnrd */
 int kill_current();
 
@@ -66,8 +109,11 @@ void cleanexit(int status);
 
 /* Reads in the domain name as a string, allocates space for the CNAME
    version of it */
-char* make_cname(const char *text);
-void sprintf_cname(const char *cname, char *buf, int bufsize);
+char* make_cname(const char *text, const int maxlen);
+void sprintf_cname(const char *cname, int namesize, char *buf, int bufsize);
+
+char *cname2asc(const char *cname);
+
 
 /* Dumping DNS packets */
 int dump_dnspacket(char *type, unsigned char *packet, int len);
