@@ -27,6 +27,7 @@
 #include <ctype.h>
 
 #include <netdb.h>
+#include <sys/types.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
@@ -35,10 +36,18 @@
 #include "lib.h"
 #include "common.h"
 
+/*
+static int get_objectname(unsigned char *msg, unsigned const char *limit, 
+			  unsigned char **here, char *string, int strlen,
+			  int k);
+*/
 
-static int get_objectname(unsigned char *msg, unsigned char **here,
-			  char *string, int k);
-
+/*
+static int get_objname(unsigned char buf[], const int bufsize, int *here,
+		      char name[], const int namelen) {
+  if (*here > bufsize) return 0;
+}
+*/
 
 int free_packet(dnsheader_t *x)
 {
@@ -51,10 +60,10 @@ static dnsheader_t *alloc_packet(void *packet, int len)
 {
     dnsheader_t *x;
 
-    x = malloc(sizeof(dnsheader_t));
-    memset(x, 0, sizeof(dnsheader_t));
+    x = allocate(sizeof(dnsheader_t));
+    /*    memset(x, 0, sizeof(dnsheader_t));*/
 
-    x->packet = malloc(len + 2);
+    x->packet = allocate(len + 2);
     x->len    = len;
     memcpy(x->packet, packet, len);
 
@@ -109,43 +118,73 @@ static int raw_dump(dnsheader_t *x)
     return (0);
 }
 
-static int get_objectname(unsigned char *msg, unsigned char **here,
-			  char *string, int k)
+/*
+static int get_objname(unsigned char buf[], const int bufsize, int *here,
+		      char name[], const int namelen) {
+  int i,p=*here, count=1000;
+  unsigned int len, offs;
+  if (p > bufsize) return 0;
+  while (len = buf[p]) {
+    
+    while (len & 0x0c) {
+      if (++p > bufsize) return 0;
+      offs = lenbuf[p] 
+
+}
+*/
+
+
+static int get_objectname(unsigned char *msg, unsigned const char *limit, 
+			  unsigned char **here, char *string, int strlen,
+			  int k)
 {
     unsigned int len;
     int	i;
 
+    if ((*here>=limit) || (k>=strlen)) return(-1);
     while ((len = **here) != 0) {
-	*here += 1;
 
+	*here += 1;
+	if ( *here >= limit ) return(-1);
+	/* If the name is compressed (see 4.1.4 in rfc1035)  */
 	if (len & 0xC0) {
 	    unsigned offset;
 	    unsigned char *p;
 
 	    offset = ((len & ~0xc0) << 8) + **here;
-	    p = &msg[offset];
-	    k = get_objectname(msg, &p, string, k);
+	    if ((p = &msg[offset]) >= limit) return(-1);
+	    if (p == *here-1) {
+	      log_debug("looping ptr");
+	      return(-2);
+	    }
 
+	    if ((k = get_objectname(msg, limit, &p, string, RR_NAMESIZE, k))<0)
+	      return(-1); /* if we cross the limit, bail out */
 	    break;
 	}
 	else if (len < 64) {
-	    for (i=0; i < len; i++) {
-		string[k++] = **here;
-		*here += 1;
-	    }
+	  /* check so we dont pass the limits */
+	  if (((*here + len) > limit) || (len+k+2 > strlen)) return(-1);
 
-	    string[k++] = '.';
+	  for (i=0; i < len; i++) {
+	    string[k++] = **here;
+	    *here += 1;
+	  }
+
+	  string[k++] = '.';
 	}
     }
 
     *here += 1;
     string[k] = 0;
-
+    
     return (k);
 }
 
+
 static unsigned char *read_record(dnsheader_t *x, rr_t *y,
-				  unsigned char *here, int question)
+				  unsigned char *here, int question,
+				  unsigned const char *limit)
 {
     int	k, len;
     unsigned short int conv;
@@ -154,9 +193,13 @@ static unsigned char *read_record(dnsheader_t *x, rr_t *y,
      * Read the name of the resource ...
      */
 
-    k = get_objectname(x->packet, &here, y->name, 0);
+    k = get_objectname(x->packet, limit, &here, y->name, sizeof(y->name), 0);
+    if (k < 0) return(NULL);
     y->name[k] = 0;
 
+    /* safe to read TYPE and CLASS? */
+    if ((here+4) > limit) return (NULL);
+
     /*
      * ... the type of data ...
      */
@@ -179,10 +222,13 @@ static unsigned char *read_record(dnsheader_t *x, rr_t *y,
 
     if (question != 0) return (here);
 
+
     /*
      * ... while answer blocks carry a TTL and the actual data.
      */
 
+    /* safe to read TTL and RDLENGTH? */
+    if ((here+6) > limit) return (NULL);
     memcpy(&y->ttl, here, sizeof(unsigned long int));
     y->ttl = ntohl(y->ttl);
     here += 4;
@@ -191,9 +237,13 @@ static unsigned char *read_record(dnsheader_t *x, rr_t *y,
      * Fetch the resource data.
      */
 
-    memcpy(&y->len, here, sizeof(unsigned short int));
-    len = y->len = ntohs(y->len);
+    memcpy(&conv, here, sizeof(unsigned short int));
+    len = y->len = ntohs(conv);
     here += 2;
+
+    /* safe to read RDATA? */
+    if ((here + y->len) > limit) return (NULL);
+    
     if (y->len > sizeof(y->data) - 4) {
 	y->len = sizeof(y->data) - 4;
     }
@@ -208,53 +258,74 @@ static unsigned char *read_record(dnsheader_t *x, rr_t *y,
 
 int dump_dnspacket(char *type, unsigned char *packet, int len)
 {
-    int	i;
-    rr_t	y;
-    dnsheader_t *x;
+  int	i;
+  rr_t	y;
+  dnsheader_t *x;
+  unsigned char *limit;
 
-    if (opt_debug < 2) return 0;
-    x = decode_header(packet, len);
+  if (opt_debug < 2) return 0;
 
-    fprintf(stderr, "\n");
-    fprintf(stderr, "- -- %s\n", type);
-    raw_dump(x);
+  if ((x = decode_header(packet, len)) == NULL ) {
+    return (-1);
+  }
+  limit = x->packet + len;
 
-    fprintf(stderr, "\n");
-    fprintf(stderr, "id= %u, q= %d, opc= %d, aa= %d, wr/ra= %d/%d, "
-	     "trunc= %d, rcode= %d [%04X]\n",
-	     x->id, GET_QR(x->u), GET_OPCODE(x->u), GET_AA(x->u),
-	     GET_RD(x->u), GET_RA(x->u), GET_TC(x->u), GET_RCODE(x->u), x->u);
+  if (x->u & MASK_Z) log_debug("Z is set");
 
-    fprintf(stderr, "qd= %u\n", x->qdcount);
-    x->here = read_record(x, &y, x->here, 1);
-    fprintf(stderr, "  name= %s, type= %d, class= %d\n",
-	     y.name, y.type, y.class);
-    
-    fprintf(stderr, "ans= %u\n", x->ancount);
-    for (i = 0; i < x->ancount; i++) {
-	x->here = read_record(x, &y, x->here, 0);
-	fprintf(stderr, "  name= %s, type= %d, class= %d, ttl= %lu\n",
-		 y.name, y.type, y.class, y.ttl);
-    }
-	    
-    fprintf(stderr, "ns= %u\n", x->nscount);
-    for (i = 0; i < x->nscount; i++) {
-	x->here = read_record(x, &y, x->here, 0);
-	fprintf(stderr, "  name= %s, type= %d, class= %d, ttl= %lu\n",
-		 y.name, y.type, y.class, y.ttl);
-    }
-	    
-    fprintf(stderr, "ar= %u\n", x->arcount);
-    for (i = 0; i < x->arcount; i++) {
-	x->here = read_record(x, &y, x->here, 0);
-	fprintf(stderr, "  name= %s, type= %d, class= %d, ttl= %lu\n",
-		 y.name, y.type, y.class, y.ttl);
-    }
-	    
-    fprintf(stderr, "\n");
+  fprintf(stderr, "\n");
+  fprintf(stderr, "- -- %s\n", type);
+  raw_dump(x);
+  
+  fprintf(stderr, "\n");
+  fprintf(stderr, "id= %u, q= %d, opc= %d, aa= %d, wr/ra= %d/%d, "
+	  "trunc= %d, rcode= %d [%04X]\n",
+	  x->id, GET_QR(x->u), GET_OPCODE(x->u), GET_AA(x->u),
+	  GET_RD(x->u), GET_RA(x->u), GET_TC(x->u), GET_RCODE(x->u), x->u);
+
+  fprintf(stderr, "qd= %u\n", x->qdcount);
+  
+  if ((x->here = read_record(x, &y, x->here, 1, limit)) == NULL) {
     free_packet(x);
+    return(-1);
+  }
 
-    return (0);
+  fprintf(stderr, "  name= %s, type= %d, class= %d\n",
+	  y.name, y.type, y.class);
+  
+  fprintf(stderr, "ans= %u\n", x->ancount);
+  for (i = 0; i < x->ancount; i++) {
+    if ((x->here = read_record(x, &y, x->here, 0, limit)) == NULL) {
+      free_packet(x);
+      return(-1);
+    }
+    fprintf(stderr, "  name= %s, type= %d, class= %d, ttl= %lu\n",
+	    y.name, y.type, y.class, y.ttl);
+  }
+	    
+  fprintf(stderr, "ns= %u\n", x->nscount);
+  for (i = 0; i < x->nscount; i++) {
+    if ((x->here = read_record(x, &y, x->here, 0, limit))==NULL) {
+      free_packet(x);
+      return(-1);
+    }
+    fprintf(stderr, "  name= %s, type= %d, class= %d, ttl= %lu\n",
+	    y.name, y.type, y.class, y.ttl);
+  }
+    
+  fprintf(stderr, "ar= %u\n", x->arcount);
+  for (i = 0; i < x->arcount; i++) {
+    if ((x->here = read_record(x, &y, x->here, 0, limit))==NULL) {
+      free_packet(x);
+      return(-1);
+    }
+    fprintf(stderr, "  name= %s, type= %d, class= %d, ttl= %lu\n",
+	    y.name, y.type, y.class, y.ttl);
+  }
+	    
+  fprintf(stderr, "\n");
+  free_packet(x);
+
+  return (0);
 }
 
 
@@ -267,6 +338,7 @@ dnsheader_t *parse_packet(unsigned char *packet, int len)
     return (x);
 }
 
+/*
 int get_dnsquery(dnsheader_t *x, rr_t *query)
 {
     char	*here;
@@ -280,6 +352,7 @@ int get_dnsquery(dnsheader_t *x, rr_t *query)
 
     return (0);
 }
+*/
 
 /*
  * parse_query()
@@ -294,17 +367,31 @@ unsigned char *parse_query(rr_t *y, unsigned char *msg, int len)
     int	k;
     unsigned char *here;
     unsigned short int conv;
+    unsigned const char *limit = msg+len; 
 
-    if (ntohs(((short int *) msg)[2]) == 0) {		/* C is nice. */
-	return (0);
+    /* If QDCOUNT, the number of entries in the question section,
+     * is zero, we just give up */
+
+    if (ntohs(((dnsheader_t *)msg)->qdcount) == 0 ) {
+      log_debug("QDCOUNT was zero");
+      return(0);
     }
 
+    /*
+      if (ntohs(((dnsheader_t *)msg)->u) & MASK_QR ) {
+      return(0);
+      }
+    */
+    
     y->flags = ntohs(((short int *) msg)[1]);
 
     here = &msg[PACKET_DATABEGIN];
-    k = get_objectname(msg, &here, y->name, 0);
+    if ((k=get_objectname(msg, limit, &here, y->name, sizeof(y->name),0))< 0) 
+      return(0);
     y->name[k] = 0;
 
+    /* check that there is type and class */
+    if (here + 4 > limit) return(0);
     memcpy(&conv, here, sizeof(unsigned short int));
     y->type = ntohs(conv);
     here += 2;
@@ -313,12 +400,16 @@ unsigned char *parse_query(rr_t *y, unsigned char *msg, int len)
     y->class = ntohs(conv);
     here += 2;
 
+    /* those strings should have been checked in get_objectname */
     k = strlen(y->name);
     if (k > 0  &&  y->name[k-1] == '.') {
 	y->name[k-1] = '\0';
     }
 
-    strlwr(y->name);
+    /* should we really convert the name to lowercase?  
+     * rfc1035 2.3.3
+     */
+    strnlwr(y->name, sizeof(y->name));
 	    
     return (here);
 }
