@@ -37,6 +37,7 @@
 
 /* Macros for manipulating the flags field */
 #define MASK_RCODE  0x000f
+#define MASK_Z      0x0070
 #define MASK_RA     0x0080
 #define MASK_RD     0x0100
 #define MASK_TC     0x0200
@@ -45,6 +46,7 @@
 #define MASK_QR     0x8000
 
 #define GET_RCODE(x)    ((x) & MASK_RCODE)
+#define GET_Z(x)        (((x) & MASK_Z) >> 4)
 #define GET_RA(x)       (((x) & MASK_RA) >> 7)
 #define GET_RD(x)       (((x) & MASK_RD) >> 8)
 #define GET_TC(x)       (((x) & MASK_TC) >> 9)
@@ -61,16 +63,18 @@
                              (((y) << 11) & MASK_OPCODE))
 #define SET_QR(x, y)        ((x) = ((x) & ~MASK_QR) | (((y) << 15) & MASK_QR))
 
+#define RR_NAMESIZE 300
+
 typedef struct _rr {
     unsigned short      flags;
-    char	  name[300];
+    char	  name[RR_NAMESIZE];
     
     unsigned int  type;
     unsigned int  class;
 
     unsigned long ttl;
     int		  len;
-    char	  data[300];
+    char	  data[RR_NAMESIZE];
 } rr_t;
 
 
@@ -97,7 +101,6 @@ int free_packet(dnsheader_t *x);
 /* static dnsheader_t *decode_header(void *packet, int len); */
 
 dnsheader_t *parse_packet(unsigned char *packet, int len);
-int get_dnsquery(dnsheader_t *x, rr_t *query);
 unsigned char *parse_query(rr_t *query, unsigned char *msg, int len);
 
 #endif /* _DNRD_DNS_H_ */
