@@ -19,9 +19,13 @@
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
 */
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 
 #include <stdlib.h>
 #include <stdio.h>
+#define _GNU_SOURCE
 #include <string.h>
 #include <ctype.h>
 #include <unistd.h>
@@ -29,18 +33,17 @@
 #include "lib.h"
 #include "common.h"
 
+/*
 #define	DEBUG(x)		
-
+*/
 
 void *allocate(size_t size)
 {
     void	*p;
-
     if ((p = malloc(size)) == NULL) {
 	log_msg(LOG_ALERT, "memory allocation error");
 	exit (1);
     }
-
     memset(p, 0, size);
     return (p);
 }
@@ -56,27 +59,25 @@ void *reallocate(void *p, size_t size)
 }
 
 
-char *strlwr(char *string)
+char *strnlwr(char *string, const int maxlen)
 {
-    unsigned int c;
-    unsigned char *p;
+    int i;
 
-    p = string;
-    while ((c = *p) != 0) {
-	*p++ = tolower(c);
+    if (!string) return NULL;
+    for (i=0; (i<=maxlen) && (string[i] != 0); i++) {
+	string[i] = tolower(string[i]);
     }
 
     return (string);
 }	
 
-char *strupr(char *string)
+char *strnupr(char *string, const int maxlen)
 {
-    unsigned int c;
-    unsigned char *p;
+    int i;
 
-    p = string;
-    while ((c = *p) != 0) {
-	*p++ = toupper(c);
+    if (!string) return NULL;
+    for (i=0; (i<=maxlen) && string[i]; i++) {
+	string[i] = toupper(string[i]);
     }
 
     return (string);
@@ -94,7 +95,7 @@ char *skip_ws(char *string)
     return (string);
 }
 
-char *noctrl(char *buffer)
+char *noctrln(char *buffer, const int maxlen)
 {
     int	len, i;
     unsigned char *p;
@@ -103,7 +104,7 @@ char *noctrl(char *buffer)
 	return (NULL);
     }
 
-    len = strlen(p);
+    len = strnlen(p, maxlen);
     for (i=len-1; i>=0; i--) {
 	if (p[i] <= 32) {
 	    p[i] = '\0';
@@ -168,16 +169,16 @@ char *get_quoted(char **from, int delim, char *to, int max)
 
 char *copy_string(char *y, char *x, int len)
 {
-    x = skip_ws(x);
-    noctrl(x);
+    char *z = skip_ws(x);
+    noctrln(z, len - (x-z));
 
     len -= 2;
-    if (strlen(x) >= len) {
-	x[len] = 0;
+    if (strlen(z) >= len) {
+	z[len] = 0;
     }
 
-    if (y != x) {
-	strcpy(y, x);
+    if (y != z) {
+	strcpy(y, z);
     }
 	    
     return (y);
@@ -202,3 +203,36 @@ unsigned int get_stringcode(char *string)
     return (code);
 }
 
+/* in case we dont have strnlen */
+#ifndef HAVE_STRNLEN
+size_t strnlen(const char *s, size_t maxlen) {
+  size_t len=0;
+  while (*s++ && len<maxlen) len++;
+  return (len);
+}
+#endif
+
+#ifndef HAVE_USLEEP
+/*
+
+  usleep -- support routine for 4.2BSD system call emulations
+  last edit:  29-Oct-1984     D A Gwyn
+*/
+
+extern int        select();
+
+int usleep( usec )              /* returns 0 if ok, else -1 */
+     long       usec;           /* delay in microseconds */
+{
+  static struct                 /* `timeval' */
+  {
+    long        tv_sec;         /* seconds */
+    long        tv_usec;        /* microsecs */
+  }   delay;                    /* _select() timeout */
+  
+  delay.tv_sec = usec / 1000000L;
+  delay.tv_usec = usec % 1000000L;
+  
+  return select( 0, (long *)0, (long *)0, (long *)0, &delay );
+}
+#endif
