@@ -23,6 +23,9 @@
 #ifndef	_LIB_INCLUDED
 #define	_LIB_INCLUDED
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 
 extern char *program;
 extern int verbose;
@@ -31,16 +34,24 @@ extern int verbose;
 void *allocate(size_t size);
 void *reallocate(void *p, size_t size);
 
-char *strlwr(char *string);
-char *strupr(char *string);
+char *strnlwr(char *string, const int maxlen);
+char *strnupr(char *string, const int maxlen);
 
 char *skip_ws(char *string);
-char *noctrl(char *buffer);
+char *noctrln(char *buffer, const int maxlen);
 char *get_word(char **from, char *to, int maxlen);
 char *get_quoted(char **from, int delim, char *to, int max);
 char *copy_string(char *y, char *x, int len);
 
 unsigned int get_stringcode(char *string);
 
+#ifndef HAVE_STRNLEN
+size_t strnlen(const char *s, size_t maxlen);
+#endif
+
+#ifndef HAVE_USLEEP
+int usleep(long usec);
+#endif
+
 #endif
 
