@@ -18,14 +18,9 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#include "relay.h"
-#include "cache.h"
-#include "common.h"
-#include "args.h"
-#include "sig.h"
-#include "master.h"
-#include <stdio.h>
 #include <sys/types.h>
+#include <stdio.h>
+#include <stdlib.h>
 #include <sys/stat.h>
 #include <sys/socket.h>
 #include <sys/stat.h>
@@ -39,6 +34,15 @@
 #include <pwd.h>
 #include <dirent.h>
 
+#include "relay.h"
+#include "cache.h"
+#include "common.h"
+#include "args.h"
+#include "sig.h"
+#include "master.h"
+#include "domnode.h"
+#include "lib.h"
+
 /*
  * main() - startup the program.
  *
@@ -53,7 +57,7 @@
  */
 int main(int argc, char *argv[])
 {
-    int                i;
+  /*    int                i;*/
     FILE              *filep;
     struct servent    *servent;   /* Let's be good and find the port numbers
 				     the right way */
@@ -62,6 +66,10 @@ int main(int argc, char *argv[])
     struct dirent     *direntry;
     struct stat        st;
     int                rslt;
+    domnode_t *p;
+    srvnode_t *s;
+
+
 
     /*
      * Setup signal handlers.
@@ -78,6 +86,19 @@ int main(int argc, char *argv[])
      */
     signal(SIGCHLD, SIG_IGN);
 
+    /*
+     * Initialization in common.h of recv_addr is broken, causing at
+     * least the '-a' switch not to work.  Instead of assuming
+     * positions of fields in the struct across platforms I thought it
+     * safer to do a standard initialization in main().
+     */
+    memset(&recv_addr, 0, sizeof(recv_addr));
+    recv_addr.sin_family = AF_INET;
+    recv_addr.sin_port = htons(53);
+
+
+    /* create the domain list */
+    domain_list = alloc_domnode();
     /*
      * Parse the command line.
      */
@@ -140,6 +161,7 @@ int main(int argc, char *argv[])
     /*
      * Setup our DNS tcp proxy socket.
      */
+#ifdef ENABLE_TCP
     if ((tcpsock = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
 	log_msg(LOG_ERR, "tcpsock: Couldn't open socket");
 	cleanexit(-1);
@@ -156,12 +178,18 @@ int main(int argc, char *argv[])
 	log_msg(LOG_ERR, "tcpsock: Can't listen");
 	cleanexit(-1);
     }
+#endif
 
     /* Initialise our cache */
     cache_init();
 
+    /* init the qid pool */
+    qid_init_pool();
+
+#ifndef EXCLUDE_MASTER
     /* Initialise out master DNS */
     master_init();
+#endif
 
     pwent = getpwnam("nobody");
 
@@ -169,10 +197,11 @@ int main(int argc, char *argv[])
      * Change our root and current working directories to /etc/dnrd.
      * Also, so some sanity checking on that directory first.
      */
-    dirp = opendir("/etc/dnrd");
+    dirp = opendir(chroot_path);
     if (!dirp) {
-	log_msg(LOG_ERR, "The directory /etc/dnrd must be created before "
-		"dnrd will run");
+	log_msg(LOG_ERR, "The directory %s must be created before "
+		"dnrd will run", chroot_path);
+	cleanexit(-1);
     }
 
     rslt = stat("/etc/dnrd", &st);
@@ -214,14 +243,14 @@ int main(int argc, char *argv[])
     }
     closedir(dirp);
 
-    if (chdir("/etc/dnrd")) {
+    if (chdir(chroot_path)) {
 	log_msg(LOG_ERR, "couldn't chdir to %s, %s",
-		"/etc/dnrd", strerror(errno));
+		chroot_path, strerror(errno));
 	cleanexit(-1);
     }
-    if (chroot("/etc/dnrd")) {
+    if (chroot(chroot_path)) {
 	log_msg(LOG_ERR, "couldn't chroot to %s, %s",
-		"/etc/dnrd", strerror(errno));
+		chroot_path, strerror(errno));
 	cleanexit(-1);
     }
 
@@ -277,15 +306,20 @@ int main(int argc, char *argv[])
     /*
      * Setup our DNS query forwarding socket.
      */
-    for (i = 0; i < serv_cnt; i++) {
-	if ((dns_srv[i].sock = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
+    p=domain_list;
+    do {
+      s=p->srvlist;
+      while ((s=s->next) != p->srvlist) {
+	if ((s->sock = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
 	    log_msg(LOG_ERR, "osock: Couldn't open socket");
 	    cleanexit(-1);
 	}
-
-	dns_srv[i].addr.sin_family = AF_INET;
-	dns_srv[i].addr.sin_port   = htons(53);
-    }
+	s->addr.sin_family = AF_INET;
+	s->addr.sin_port   = htons(53);
+      }
+      /* set the first server as current */
+      p->current = p->srvlist->next;
+    } while ((p=p->next) != domain_list);
 
     /*
      * Now it's time to become a daemon.
