@@ -18,12 +18,19 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
+#ifdef ENABLE_TCP
+#ifdef ENABLE_PTHREADS
+#include <pthread.h>
+#endif
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 #include <unistd.h>
 #include <signal.h>
-#include <wait.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/fcntl.h>
@@ -36,9 +43,15 @@
 #include "common.h"
 #include "relay.h"
 #include "cache.h"
-#include "pthread.h"
+#include "lib.h"
 
 
+#ifdef ENABLE_PTHREADS
+#define TCP_EXIT(x) pthread_exit((x))
+#else
+#define TCP_EXIT(x) _exit((x))
+#endif
+
 typedef struct tcp_handle_info {
     int	               connect;
     struct sockaddr_in client;
@@ -91,25 +104,32 @@ static int ip_open(struct sockaddr_in server, unsigned int port)
  *                 proxying.  After the request is served the thread
  *                 terminates.
  */
-static void *tcp_handler(void *dummy)
+static void *tcp_handler(tcp_handle_t *dummy)
 {
     tcp_handle_t      *arg = (tcp_handle_t *)dummy;
     int	               rc, bytes;
     int                maxsock;
-    int                server[MAX_SERV];
-    char	       buffer[4096];
+    //    int                server[MAX_SERV];
+    char	       buffer[4096]; /* FIXME: do we want this on the stack?*/
     struct timeval     tov;
     fd_set	       connection, available;
     unsigned short     tcpsize;
-    int                i;
     int	               connect = arg->connect;
     struct sockaddr_in client  = arg->client;
+    domnode_t *d =domain_list;
+    srvnode_t *s;
 
-    free(arg);
+    /*    free(arg);*/
 
+    do {
+      if ((s=d->srvlist))
+	while ((s=s->next) != d->srvlist) s->tcp = -1;
+    } while ((d=d->next) != domain_list);
+    /*
     for(i = 0; i < MAX_SERV; i++) {
 	server[i] = -1;
     }
+    */
 
     maxsock = connect + 1;
     FD_ZERO(&connection);
@@ -125,14 +145,15 @@ static void *tcp_handler(void *dummy)
 	rc = select(maxsock, &available, NULL, NULL, &tov);
 	if (rc < 0) {
 	    log_msg(LOG_ERR, "[%d] select() error: %m\n", getpid());
-	    pthread_exit(0);
+	    TCP_EXIT(0);
 	}
 	else if (rc == 0) {
 	    log_msg(LOG_NOTICE, "[%d] connection timed out", getpid());
-	    pthread_exit(0);
+	    TCP_EXIT(0);
 	}
 
 	/* Forward replies from DNS server to client */
+	/*  
 	for(i = 0; i < serv_cnt; i++) {
 	    if (server[i] == -1) continue;
 	    if (FD_ISSET(server[i], &available)) {
@@ -149,12 +170,36 @@ static void *tcp_handler(void *dummy)
 		}
 	    }
 	}
+*/
+	d=domain_list;
+	do {
+	  if ((s=d->srvlist)) 
+	    while ((s=s->next) != d->srvlist) {
+	      if (s->tcp == -1) continue;
+	      if (FD_ISSET(s->tcp, &available)) {
+		log_debug("[%d] Received tcp reply.  Forwarding...", getpid());
+		if ((bytes = read(s->tcp, buffer, sizeof(buffer))) <= 0) {
+		  child_die = 1;
+		  break;
+		}
+		dump_dnspacket("reply", buffer + 2, bytes - 2);
+		cache_dnspacket(buffer + 2, bytes - 2, s);
+		if (write(connect, buffer, bytes) != bytes) {
+		  child_die = 1;
+		  break;
+		}
+	      }
+	    }
+	  if (child_die) break;
+	} while ((d=d->next) != domain_list);
+	  
 	if (child_die) break;
 
 	/* Forward requests from client to DNS server */
 	if (FD_ISSET(connect, &available)) {
 	    int retn;
-	    unsigned srvridx;
+	    domnode_t *dptr;
+	    
 
 	    bytes = read(connect, &tcpsize, sizeof(tcpsize));
 	    /* check for connection close */
@@ -182,10 +227,12 @@ static void *tcp_handler(void *dummy)
 		log_debug("[%d] tcp read error %s", getpid(), strerror(errno));
 		break;
 	    }
-	    retn = handle_query(&client, &buffer[2], &bytes, &srvridx);
-
+	    if ((retn = handle_query(&client, buffer + 2, &bytes, &dptr))<0) {
+	      /* bogus query */
+	      break;
+	    }
 	    /* If we can reply locally (master, cache, no servers), do so. */
-	    if (retn == 0) {
+	    else if (retn == 0) {
 		unsigned short len = htons((unsigned short)bytes);
 		memcpy(buffer, &len, sizeof(len));
 		if (write(connect, buffer, bytes + 2) != (bytes + 2)) break;
@@ -195,33 +242,43 @@ static void *tcp_handler(void *dummy)
 	     * Forward DNS request to the appropriate server.
 	     * Open a socket if one doesn't already exist.
 	     */
-	    else {
-		if (server[srvridx] == -1) {
-		    server[srvridx] = ip_open(dns_srv[srvridx].addr, 53);
-		    if (server[srvridx] < 0) {
-			log_msg(LOG_ERR, "[%d] Can't connect to server",
-				getpid());
-			break;
-		    }
-		    if (server[srvridx] > (maxsock - 1)) {
-			maxsock = server[srvridx] + 1;
-		    }
-		    FD_SET(server[srvridx], &connection);
+	    else if (retn == 1) {
+	      s = dptr->current;
+	      if (s->tcp == -1) {
+		s->tcp = ip_open(s->addr, 53);
+		if (s->tcp < 0) {
+		  log_msg(LOG_ERR, "[%d] Can't connect to server",
+			  getpid());
+		  break;
 		}
-		if (write(server[srvridx], buffer, bytes + 2) != (bytes + 2)) {
-		    break;
+		if (s->tcp > (maxsock - 1)) {
+		  maxsock = s->tcp + 1;
 		}
+		FD_SET(s->tcp, &connection);
+	      }
+	      if (write(s->tcp, buffer, bytes + 2) != (bytes + 2)) {
+		break;
+	      }
 	    }
 	}
-
+	
     }
     /* The child process is done.  It can now die */
     log_debug("[%d] Closing tcp connection", getpid());
-    for(i = 0; i < MAX_SERV; i++) {
+    /*    for(i = 0; i < MAX_SERV; i++) {
 	if (server[i] != -1) close(server[i]);
     }
+    */
+    d=domain_list;
+    do {
+      if ((s=d->srvlist))
+	while ((s=s->next) != d->srvlist) 
+	  if (s->tcp != -1) close(s->tcp);
+    } while ((d=d->next) != domain_list);
+
     close(connect);
-    pthread_exit(0);
+    log_debug("[%d] Exiting child", getpid());
+    TCP_EXIT(0);
     /* NOTREACHED */
     return (void *)0;
 }
@@ -233,22 +290,45 @@ static void *tcp_handler(void *dummy)
  * A thread is used instead of a process so that any DNS replies can
  * be placed in cache of the parent process.
  */
+
+
 void handle_tcprequest()
 {
     tcp_handle_t *arg;
+#ifdef ENABLE_PTHREADS
     pthread_t t;
+    pthread_attr_t  attr;
+#endif
+    pid_t pid;
 
-    arg = (tcp_handle_t *)malloc(sizeof(tcp_handle_t));
+    arg = (tcp_handle_t *)allocate(sizeof(tcp_handle_t));
     arg->len = sizeof(arg->client);
 
     arg->connect = accept(tcpsock, (struct sockaddr *) &(arg->client), 
 			  &(arg->len));
     if (arg->connect < 0) {
-	log_msg(LOG_ERR, "accept error: %m");
+	log_msg(LOG_ERR, "accept error for TCP connection");
+	free(arg);
 	return;
     }
 
-    if (pthread_create(&t, NULL, tcp_handler, (void *)arg)) {
+#ifdef ENABLE_PTHREADS
+    pthread_attr_init (&attr);
+    pthread_attr_setdetachstate (&attr, PTHREAD_CREATE_DETACHED);
+    log_debug("creating a thread (parent=%d)", getpid());
+    if (pthread_create(&t, &attr, (void *)&tcp_handler, (void *)arg)) {
 	log_msg(LOG_ERR, "Couldn't spawn thread to handle tcp connection");
+	free(arg);
     }
+#else
+    if ((pid=fork())==0) {
+      /* we are the child */
+      tcp_handler(arg);
+    } else if (pid < 0) {
+      log_msg(LOG_ERR, "fork for TCP connection failed");
+    }
+    free(arg);
+#endif
+    
 }
+#endif
