@@ -20,12 +20,16 @@
 
 #ifndef _DNRD_UDP_H_
 #define _DNRD_UDP_H_
-
+#include "srvnode.h"
 
 /* Function to call when a message is available on isock */
 void handle_udprequest();
 
 /* Call this to handle upd DNS replies */
-void handle_udpreply(int srvidx);
+void handle_udpreply(srvnode_t *srv);
+
+/* send a reactivation packet */
+int send_dummy(srvnode_t *s);
+
 
 #endif /* _DNRD_UDP_H_ */
